/*
 * Ball.java

    evMiniPool	-	multiplayer pool game engine for mobile phones
    Copyright (C) 2005, Ernestas Vaiciukevicius (ernisv@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *
 * Created on Antradienis, 2003, Vasario 25, 14.15
 */

package com.ev.minipool;

import javax.microedition.lcdui.*;

/**
 * @author Ernestas Vaiciukevicius 
 *
 */
public class Ball extends Object {
    protected Point position, oldPosition;
    protected Point velVect;
    public int radius;
    public int color;
    public static final Object lock = new Object();
    protected BaseBoard board;

    /** Creates new Ball */
    public Ball(BaseBoard board, int radius, int color) {
    	this.radius = radius;
        this.board = board;
        this.color = color;
        position = new Point(0, 0);
        oldPosition = new Point(0, 0);
        velVect = new Point(0, 0);
    }
    
    public Ball(Point pos, int color, BaseBoard board, int radius) {
    	this.radius = radius;
        this.board = board;
        this.color = color;
        position = pos;
        oldPosition = new Point(0, 0);
        oldPosition.x = pos.x;
        oldPosition.y = pos.y;
        velVect = new Point(0, 0);
    }

    public void moveTo(Point pos) {
        oldPosition = position;
        position = pos;
    }

    public void moveBall() {
        Point newPos = new Point(position).addVector(velVect);
        if (!board.inField(newPos)) {
            if (newPos.x < 0 || newPos.x > board.getBounds().x-radius) velVect.x = -velVect.x;
            if (newPos.y < 0 || newPos.y > board.getBounds().y-radius) velVect.y = -velVect.y;
        } else {
            oldPosition = position;
            position = newPos;
        };
       	velVect.x = velVect.x * 995 / 1000;
        velVect.y = velVect.y * 995 / 1000;
    }
    
    public void setVelocityVect(Point vel) {
        velVect.x = vel.x;
        velVect.y = vel.y;
//        System.out.println("SetVelocityVect:Free:"+Runtime.getRuntime().freeMemory());
    }

    public Point getVelocityVect() {
        return velVect;
    }

    public void setPosition(Point pos) {
        position = pos;
    }

    public Point getPosition() {
        return position;
    }

    public Point getOldPosition() {
        return oldPosition;
    }
    
    public boolean isIntersecting(Point a, Point b) {
    	return isIntersecting(a, b, radius);
    }

    // is balls at points a and b intersecting?
    public static boolean isIntersecting(Point a, Point b, int radius) {
        return (Math.max(a.x, b.x)-Math.min(a.x, b.x) < radius) &&
               (Math.max(a.y, b.y)-Math.min(a.y, b.y) < radius);
    }

    public static void paint(Graphics g, int color, int x, int y, int radius, int scale) {
    	g.setColor(color);
        g.fillArc((x*scale)>>8, (y*scale)>>8, ((radius*scale)>>8), ((radius*scale)>>8), 0, 360);
        g.setColor(0x000000);
        g.drawArc((x*scale)>>8, (y*scale)>>8, ((radius*scale)>>8), ((radius*scale)>>8), 0, 360);
    }

    public void paint(Graphics g, int scale) {
        paint(g, color, position.x, position.y, radius, scale);
    }

	public int getColor() {
		return color;
	}

	public void setColor(int color) {
		this.color = color;
	}
}
