/*
 * Board.java

    evMiniPool	-	multiplayer pool game engine for mobile phones
    Copyright (C) 2005, Ernestas Vaiciukevicius (ernisv@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *
 * Created on Ketvirtadienis, 2003, Vasario 27, 12.09
 */

package com.ev.minipool;

import java.util.Vector;
import javax.microedition.midlet.*;

/**
 * @author Ernestas Vaiciukevicius 
 *
 */
public class Board extends BaseBoard implements Runnable {

    private final static int maxBalls = 16;
    protected boolean wasMovement;
    protected Vector trajectory;
    protected int notifyNum;
    protected MIDlet midlet;
    public int ballsFalled;

  /** Creates new Board */
    public Board(Point bounds, MIDlet midlet, int ballRadius) {
        super(new Point(bounds.x, bounds.y), ballRadius);
        this.midlet = midlet;
        trajectory = new Vector();
        init();
    }

    public void init() {
        trajectory = new Vector();
        ballsFalled = 0;
        balls = new Ball[maxBalls];
        
        int hMiddle = bounds.x / 2;
        int triangleCenter = hMiddle / 2 + ballRadius/2;
        
        int vMiddle = bounds.y / 2;
        
        balls[0] = new Ball(new Point(hMiddle + hMiddle / 2, vMiddle), 0x000000, this, getBallRadius());
        
        balls[1] = new Ball(new Point(triangleCenter + 2*ballRadius, vMiddle), 0xff0000, this, getBallRadius());
        balls[2] = new Ball(new Point(triangleCenter + ballRadius, vMiddle - ballRadius/2), 0x00ff00, this, getBallRadius());
        balls[3] = new Ball(new Point(triangleCenter + ballRadius, vMiddle + ballRadius/2), 0x0000ff, this, getBallRadius());
        balls[4] = new Ball(new Point(triangleCenter, vMiddle), 0xffff00, this, getBallRadius());
        balls[5] = new Ball(new Point(triangleCenter, vMiddle-ballRadius), 0x00ffff, this, getBallRadius());
        balls[6] = new Ball(new Point(triangleCenter, vMiddle+ballRadius), 0xf0f000, this, getBallRadius());
        balls[7] = new Ball(new Point(triangleCenter - ballRadius, vMiddle-ballRadius/2), 0x0000f0, this, getBallRadius());
        balls[8] = new Ball(new Point(triangleCenter - ballRadius, vMiddle-ballRadius-ballRadius/2), 0x0f00f0, this, getBallRadius());
        balls[9] = new Ball(new Point(triangleCenter - ballRadius, vMiddle+ballRadius/2), 0xf00000, this, getBallRadius());
        balls[10] = new Ball(new Point(triangleCenter-ballRadius, vMiddle+ballRadius+ballRadius/2), 0x00f000, this, getBallRadius());
    }

    public boolean wasMovement() {
        return wasMovement;
    }

    public void notifyAfter(int n) {
    		notifyNum = n;
    }
    
    public void checkHoles() {
        for (int i = 0; i < balls.length; ++i) 
            if (balls[i] != null) {
                Point pos = new Point(balls[i].getPosition()).addVector(balls[i].getVelocityVect());
                pos.x += getBallRadius() >> 1;
                pos.y += getBallRadius() >> 1;
//System.out.println("Bounds:"+pos);
//System.out.println("Pos:"+pos);
                if (pos.y >= getBallRadius() && pos.y <= bounds.y-getBallRadius()) continue;
//System.out.println("Y OK....");
                if (pos.x < getBallRadius() || pos.x > bounds.x-getBallRadius() 
                    || Math.abs(pos.x-(bounds.x >> 1)) < getBallRadius()) {  // ball falls into hole
//System.out.println("Falled....");
                        balls[i] = null;
                        ++ballsFalled;
                }
            }
    }
    
    public void run() {
	    try {
   	 	int num = 0;
      	wasMovement = true;
         trajectory.removeAllElements();
         System.gc();
	      while (wasMovement) {
	            wasMovement = false;
                checkHoles();
	            for (int i = 0; i < balls.length; ++i)
	                if (balls[i] != null) {
	                    balls[i].moveBall();
	                    if (!balls[i].getVelocityVect().equals(Point.nullPoint)) wasMovement = true;
	                }
                bounceBalls();
        		++num;
                Point[] arr = new Point[maxBalls];
                for (int i = 0; i < balls.length; ++i)
                    if (balls[i] == null)
                        arr[i] = null;
                    else arr[i] = new Point(balls[i].getPosition());
                trajectory.addElement(arr);
                if (num >= notifyNum)
    	         	synchronized (this) {
                        notifyAll();
                    }
			};
        	synchronized (this) {
                notifyAll();
            }
      } catch (Exception exx) {
      	Error.log(exx, midlet);
      }
    }

    // if some balls bounced returns true
    public boolean bounceBalls() {
        boolean bounced = false;
        for (int a = 0; a < balls.length; ++a)
            if (balls[a] != null)
                for (int b = a+1; b < balls.length; ++b)
                    if (balls[b] != null)
                        if (balls[b].isIntersecting(balls[a].getPosition(), balls[b].getPosition())) {
                                // searching for exact collision point
                                int vel1 = (int)MyMath.dist(Point.nullPoint, balls[a].getVelocityVect()), 
                                    vel2 = (int)MyMath.dist(Point.nullPoint, balls[b].getVelocityVect());
                                
                                Point p1 = new Point(balls[a].getOldPosition()),    // collision coords
                                      p2 = new Point(balls[b].getOldPosition());
                                int maxStep = Math.max(vel1, vel2);
                                int lStep = 0, rStep = maxStep;
                                while (lStep < rStep) {
                                    Point p1tmp = new Point(0, 0),   
                                          p2tmp = new Point(0, 0);
                               		int mid = (lStep+rStep) / 2;
                                    p1tmp.x = balls[a].getOldPosition().x+balls[a].getVelocityVect().x*mid/maxStep;
                                    p1tmp.y = balls[a].getOldPosition().y+balls[a].getVelocityVect().y*mid/maxStep;
                                    p2tmp.x = balls[b].getOldPosition().x+balls[b].getVelocityVect().x*mid/maxStep;
                                    p2tmp.y = balls[b].getOldPosition().y+balls[b].getVelocityVect().y*mid/maxStep;
                                    if (balls[b].isIntersecting(p1tmp, p2tmp))
                                    	rStep = mid;
                                    else if (lStep == mid)
                                    	break;
                                    else {
                                        lStep = mid;
                                        p1.x = p1tmp.x;
                                        p1.y = p1tmp.y;
                                        p2.x = p2tmp.x;
                                        p2.y = p2tmp.y;
                                    }
                                }
                                    bounced = true;
                                    MyFloat alpha = MyMath.lineAngle(p1, p2),
                                            angle1 = MyMath.lineAngle(Point.nullPoint, balls[a].getVelocityVect()),
                                            angle2 = MyMath.lineAngle(Point.nullPoint, balls[b].getVelocityVect());
//System.out.println("a:"+new Integer(a).toString());
//System.out.println("b:"+new Integer(b).toString());
//System.out.println("oldPos1:"+balls[a].getOldPosition().toString());
//System.out.println("oldPos2:"+balls[b].getOldPosition().toString());
//System.out.println("p1:"+p1.toString());
//System.out.println("p2:"+p2.toString());
//System.out.println("vect1:"+balls[a].getVelocityVect().toString());
//System.out.println("vect2:"+balls[b].getVelocityVect().toString());
//System.out.println("alpha:"+alpha.toString());
//System.out.println("angle1:"+angle1.toString());
//System.out.println("angle2:"+angle2.toString());
//System.out.println("vel1:"+Integer.toString(vel1));
//System.out.println("vel2:"+Integer.toString(vel2));
                                    angle1.sub(alpha);
                                    angle2.sub(alpha);
                                    MyFloat collForce1 = MyMath.cos(angle1).mult(vel1),
                                            collForce2 = MyMath.cos(angle2).mult(vel2);
//System.out.println("collForce1:"+collForce1.toString());
//System.out.println("collForce2:"+collForce2.toString());
                                    
                                    Point collVect1 = new Point(MyMath.cos(alpha).mult(collForce1).toInt(), MyMath.sin(alpha).mult(collForce1).toInt()),
                                          collVect2 = new Point(MyMath.cos(alpha).mult(collForce2).toInt(), MyMath.sin(alpha).mult(collForce2).toInt());
                                            
//System.out.println("collVect1:"+collVect1.toString());
//System.out.println("collVect2:"+collVect2.toString());

                                    Point pVect = new Point(collVect1).subVector(collVect2);
//System.out.println("pVect:"+pVect.toString());
                                    balls[a].getVelocityVect().subVector(pVect);
                                    balls[b].getVelocityVect().addVector(pVect);
                                    
                                    balls[a].setPosition(p1);
                                    balls[b].setPosition(p2);
                                    
//System.out.println("newVect1:"+balls[a].getVelocityVect().toString());
//System.out.println("newVect2:"+balls[b].getVelocityVect().toString());
//System.out.println("----------------------------");

                        }

        return false;
    }

    public boolean isTrajectory() {
    	synchronized (trajectory) {
      	return !trajectory.isEmpty();
      }
    }

    public void moveTrajectory() {
 			synchronized (trajectory) {
 	   		if (!trajectory.isEmpty())
   	   		trajectory.removeElementAt(0);
     		}
    }
   
    public Point[] getBallsPos() {
    		Point ret[] = null;
        	synchronized (trajectory) {
  	      	if (!trajectory.isEmpty())
     	      	ret = (Point[])(trajectory.firstElement());
        	}
         if (ret == null) {
         	ret = new Point[maxBalls];
         	for (int i = 0; i < maxBalls; ++i)
            	if (balls[i] == null)
                 	ret[i] = null;
               else ret[i] = new Point(balls[i].getPosition());
         }
         return ret;
    }


 }
