/**

    evMiniPool	-	multiplayer pool game engine for mobile phones
    Copyright (C) 2005, Ernestas Vaiciukevicius (ernisv@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 * 
 */
package com.ev.minipool;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
/**
 * @author Ernestas Vaiciukevicius 
 *
 */
class BoardCanvas extends Canvas implements ITimeListener {
    
    // Default Color Constants
    public final int WHITE =   0xFFFFFF;
    public final int BLACK =   0x000000;
    public final int RED =     0xFF0000;
    public final int GREEN =   0x00FF00;
    public final int BLUE =    0x0000FF;
    public final int GRAY =    0xCCCCCC;
    public final int YELLOW =  0xFFFF00;
    public final int MAGENTA = 0xFF00FF;
    public final int CYAN =    0x00FFFF;
    
    public int gameState;
    protected int ballSelected;
    protected MyFloat aimingAngle;
    protected int aimingForce;
    protected int points;

    private static final int fieldWidth = 6000;
    private static final int fieldHeight = 4000;
    private static final int ballRadius =250;
    private static int boardInsetX = 4;
    private static int boardInsetY = 13;
    private static int scale;

    protected PoolMIDlet midlet;
    protected Board board;
    protected TimeThread tt;
    protected Thread evalThread;
    protected Image dblBuf;
    protected Graphics dblGr;
    protected boolean paintBusyClock;
    protected int[] ballColors;

    private int repeatCount = 1;

    /**
     *
     */
    public BoardCanvas(PoolMIDlet midlet) {
        dblBuf = Image.createImage(getWidth(), getHeight());
        dblGr = dblBuf.getGraphics();
        dblGr.setFont(Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Font.SIZE_SMALL));

        boardInsetX = 2;
    	boardInsetY = getHeight() / 5;
    	
    	int spaceTop = dblGr.getFont().getHeight() + 8; 
    	
    	scale = ((getWidth()-4) << 8) / fieldWidth;
    	scale = Math.min(scale, ((getHeight()-spaceTop-2) << 8) / fieldHeight);
    	
    	System.out.println("Width<<8:" + (getWidth() << 8) + "\nWidth:" + getWidth() + "\nHeight:" + getHeight() + "\nScale:" + scale);
    	
    	boardInsetX = Math.max(2, (getWidth() - scaled(fieldWidth)) / 2);
    	boardInsetY = spaceTop + Math.max((getHeight() - spaceTop - 2 - scaled(fieldHeight)) / 2, 0);

        this.midlet = midlet;
        paintBusyClock = false;
        aimingAngle = new MyFloat(0);
        board = new Board(new Point(fieldWidth, fieldHeight), midlet, ballRadius);
        
        ballColors = new int[board.getBalls().length];
        for (int i = 0; i < ballColors.length; ++i) {
        	if (board.getBalls()[i] != null) {
        		ballColors[i] = board.getBalls()[i].getColor();
        	}
        }

        tt = new TimeThread(50);
        tt.addTimeListener(this);
        tt.pauseTime();
        tt.start();
        gameState = GameState.STOPPED;
        points = 0;
    }
    
    public void setPoints(int p) {
        points = p;
    }
    
    public int getPoints() {
        return points;
    }
    
    public int getMovePoints() {
        return board.ballsFalled;
    }

    public void rollBalls() {
        midlet.getMultiplayerGame().suspendListeners();
        gameState = GameState.ROLLING;
        board.ballsFalled = 0;
        evalThread = new Thread(board);
        board.notifyAfter(10);
        evalThread.setPriority(Thread.NORM_PRIORITY);
        beginBusy();
        synchronized (board) {
        		evalThread.start();
            try {
            	board.wait();
            } catch (Exception ex) {};
        }
        endBusy();
        tt.resumeTime();
    }

    public void getUserMove() {
        if (gameState != GameState.BALL_SEL) {
            gameState = GameState.BALL_SEL;
            for (ballSelected = 0; ballSelected < board.getBalls().length; ++ballSelected)
                if (board.getBalls()[ballSelected] != null) break;
        } else gameState = GameState.AIMING;
        repaint();
        serviceRepaints();
    }
    
    public void showMove(int aimingForce, MyFloat angle, int ball) {
        System.gc();
//        System.out.println("ShowMove:Free before:"+Runtime.getRuntime().freeMemory());
        (board.getBalls())[ball].setVelocityVect(new Point(MyMath.cos(new MyFloat(angle)).mult(aimingForce*board.getBallRadius()/100).toInt(), 
                                              MyMath.sin(new MyFloat(angle)).mult(aimingForce*board.getMaxVelocity()/100).toInt()));
//        System.out.println("ShowMove:Free after:"+Runtime.getRuntime().freeMemory());
        rollBalls();        
    }

    public int getFieldWidth() {
        return fieldWidth;
    }

    public int getFieldHeight() {
        return fieldHeight;
    }
    
    public boolean gameEnded() {
        boolean end = true;
        Ball[] balls = board.getBalls();
        for (int i = 0; i < balls.length && end; ++i)
            end = balls[i] == null;
        return end;
    }

    protected boolean isCalculating() {
    	if (evalThread == null) return false;
      else return evalThread.isAlive();
    }

    public void timeTick() {
        if (isCalculating() || board.isTrajectory()) {
        		if (!board.isTrajectory()) {
            	while (!board.isTrajectory() && isCalculating())
               	synchronized (board) {
                  	try {
                  		board.wait(100);
                     } catch (InterruptedException ex) {};
                  }
            }
            board.moveTrajectory();
            this.repaint();
            this.serviceRepaints();
        } else {
            tt.pauseTime();
            midlet.getMultiplayerGame().resumeListeners();
            gameState = GameState.WAITING;
            midlet.moveEnded();
            repaint();
            serviceRepaints();
        }
    }

    protected void keyPressed(int keyCode) {
        repeatCount = 1;
        keyAction(keyCode);
    }

    protected void beginBusy() {
    	paintBusyClock = true;
      repaint();
      serviceRepaints();
    }

    protected void endBusy() {
    	paintBusyClock = false;
    }

    protected void keyAction(int keyCode) {
        Ball[] balls = board.getBalls();
        switch (gameState) {
            case GameState.BALL_SEL:
                int i = 0;
                midlet.getMultiplayerGame().suspendListeners();
                switch (getGameAction(keyCode)) {
                    case LEFT:
                    case DOWN:
                        i = -1;
                        break;
                    case RIGHT:
                    case UP:
                        i = 1;
                        break;
                    case FIRE:
                        System.gc();
                        aimingAngle.assign(180);
                        aimingForce = 70;
                        gameState = GameState.AIMING;
                        break;
                }
                if (i != 0)
                    for (ballSelected = (ballSelected+balls.length+i) % balls.length; 
                         balls[ballSelected] == null; 
                         ballSelected = (ballSelected+balls.length+i) % balls.length) ;
                break;
            case GameState.AIMING:
                switch (getGameAction(keyCode)) {
                    case DOWN:
                        aimingForce -= repeatCount;
                        if (aimingForce < 0) aimingForce = 0;
                        break;
                    case UP:
                        aimingForce += repeatCount;
                        if (aimingForce > 100) aimingForce = 100;
                        break;
                    case LEFT:
                        aimingAngle.sub(repeatCount);
                        break;
                    case RIGHT:
                        aimingAngle.add(repeatCount);
                        break;
                    case FIRE:
                        System.gc();
                        balls[ballSelected].setVelocityVect(new Point(MyMath.cos(new MyFloat(aimingAngle)).mult(aimingForce*board.getMaxVelocity()/100).toInt(), 
                                                                        MyMath.sin(new MyFloat(aimingAngle)).mult(aimingForce*board.getMaxVelocity()/100).toInt()));
                        midlet.userMove((byte)aimingForce, aimingAngle, (byte)ballSelected);
                        rollBalls();
                        break;
                }
                break;
        }
        repaint();
        serviceRepaints();        
    }
    
    /**
     *
     */
    protected void keyRepeated(int keyCode) {
        if (repeatCount < 30) repeatCount += 2;
        keyAction(keyCode);
    }
    
    /**
     *
     */
    protected void keyReleased(int keyCode) {
    }
    
    /**
     *
     */
    protected void pointerPressed(int x, int y) {
    }
    
    /**
     *
     */
    protected void pointerReleased(int x, int y) {
    }
    
    /** 
     *
     */
    protected void pointerDragged(int x, int y) {
    }
    
    protected int scaled(int i) {
    	return (scale * i) >> 8;
    }
    
    /**
     *
     */
    public void paint(Graphics g) {
      try {
        Point pos;
//        dblGr = g;
        dblGr.translate(-dblGr.getTranslateX(), -dblGr.getTranslateY());
        dblGr.setColor(YELLOW);
        dblGr.fillRect(0, 0, getWidth()-1, getHeight()-1);
        dblGr.setColor(WHITE);
        dblGr.fillRect(boardInsetX, boardInsetY, scaled(fieldWidth), scaled(fieldHeight));
        
        // draw board
        dblGr.setColor(BLUE);
        dblGr.fillRect(boardInsetX-2, boardInsetY-2, 2, scaled(fieldHeight)+4);
        dblGr.fillRect(boardInsetX, boardInsetY-2, scaled(fieldWidth)+2, 2);
        dblGr.fillRect(boardInsetX+scaled(fieldWidth), boardInsetY, 2, scaled(fieldHeight)+2);
        dblGr.fillRect(boardInsetX, boardInsetY+scaled(fieldHeight), scaled(fieldWidth), 2);
        // holes
        int scaledBallRadius = scaled(board.getBallRadius());
        int scaledBallRadius15 = scaledBallRadius+scaledBallRadius/2;
        
        dblGr.setColor(BLUE);
        dblGr.fillArc(boardInsetX-scaledBallRadius15/2, boardInsetY - scaledBallRadius15/2, scaledBallRadius15, scaledBallRadius15, 270, 90);
        dblGr.fillArc(boardInsetX-scaledBallRadius15/2, boardInsetY + scaled(fieldHeight) - scaledBallRadius15/2, scaledBallRadius15, scaledBallRadius15, 0, 90);
        dblGr.fillArc(boardInsetX + scaled(fieldWidth) - scaledBallRadius15/2, boardInsetY - scaledBallRadius15/2, scaledBallRadius15, scaledBallRadius15, 180, 90);
        dblGr.fillArc(boardInsetX + scaled(fieldWidth) - scaledBallRadius15/2, boardInsetY + scaled(fieldHeight) - scaledBallRadius15/2, scaledBallRadius15, scaledBallRadius15, 90, 90);
        dblGr.fillArc(boardInsetX + scaled(fieldWidth)/2 - scaledBallRadius/2, boardInsetY-scaledBallRadius/2, scaledBallRadius, scaledBallRadius, 180, 180);
        dblGr.fillArc(boardInsetX + scaled(fieldWidth)/2 - scaledBallRadius/2, boardInsetY+scaled(fieldHeight)-scaledBallRadius/2, scaledBallRadius, scaledBallRadius, 0, 180);
        
        dblGr.setColor(BLUE);
        dblGr.drawString("Score:" + String.valueOf(points), 20, 0, Graphics.TOP | Graphics.LEFT);
        dblGr.setColor(BLACK);
        dblGr.translate(boardInsetX, boardInsetY);
        Point balls[] = board.getBallsPos();
        for (int i = 0; i < balls.length; ++i) {
            if (balls[i] != null) {
                Ball.paint(dblGr, ballColors[i], balls[i].x, balls[i].y, board.getBallRadius(), scale);
            }
        }
        
        switch (gameState) {
            case GameState.BALL_SEL:
                pos = balls[ballSelected];
                dblGr.setColor(RED);
//                dblGr.drawArc(scaled(pos.x)-scaledBallRadius/2, scaled(pos.y)-scaledBallRadius/2, 2*scaledBallRadius, 2*scaledBallRadius, 0, 360);
                dblGr.drawLine(scaled(pos.x)-scaledBallRadius/2, scaled(pos.y)-scaledBallRadius/2, scaled(pos.x)+scaledBallRadius15, scaled(pos.y)+scaledBallRadius15);
                dblGr.drawLine(scaled(pos.x)-scaledBallRadius/2, scaled(pos.y)+scaledBallRadius15, scaled(pos.x)+scaledBallRadius15, scaled(pos.y)-scaledBallRadius/2);
                break;
            case GameState.AIMING:
                pos = balls[ballSelected];
                dblGr.setColor(0x000000);
                dblGr.drawLine(scaled(pos.x)+scaledBallRadius/2, 
                		scaled(pos.y)+scaledBallRadius/2, 
                		scaled(pos.x)+scaledBallRadius/2-MyMath.cos(new MyFloat(aimingAngle).sub(2)).mult(scaledBallRadius*5).toInt(), 
                		scaled(pos.y)+scaledBallRadius/2-MyMath.sin(new MyFloat(aimingAngle).sub(2)).mult(scaledBallRadius*5).toInt());
                dblGr.drawLine(scaled(pos.x)+scaledBallRadius/2, 
                		scaled(pos.y)+scaledBallRadius/2, 
                		scaled(pos.x)+scaledBallRadius/2-MyMath.cos(new MyFloat(aimingAngle).add(2)).mult(scaledBallRadius*5).toInt(), 
                		scaled(pos.y)+scaledBallRadius/2-MyMath.sin(new MyFloat(aimingAngle).add(2)).mult(scaledBallRadius*5).toInt());
                dblGr.fillRect(0, -7, scaled(aimingForce*fieldWidth/100), 4);
                break;
            case GameState.WAITING:
                dblGr.drawString("Waiting...", getWidth()/2, 0, g.HCENTER | g.TOP);
                break;
        }
        if (paintBusyClock) {
            dblGr.drawString("Free:"+Runtime.getRuntime().freeMemory(), getWidth()/2, 20, g.HCENTER | g.TOP);
        }
        
        g.translate(-g.getTranslateX(), -g.getTranslateY());
        g.drawImage(dblBuf, 0, 0, g.TOP | g.LEFT);
    } catch (Exception ex) {
  		ex.printStackTrace();
    }
  }
  
}

class GameState {
    public static final int STOPPED = 0;
    public static final int ROLLING = 1;
    public static final int BALL_SEL = 2;
    public static final int AIMING = 3;
    public static final int WAITING = 4;
}
