/*
 * MyFloat.java

    evMiniPool	-	multiplayer pool game engine for mobile phones
    Copyright (C) 2005, Ernestas Vaiciukevicius (ernisv@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *
 * Created on Penktadienis, 2003, Vasario 28, 13.22
 */

package com.ev.minipool;

/**
 * @author Ernestas Vaiciukevicius 
 *
 */
public class MyFloat extends Object {
    public static final int MULTIPL = 1000;
    public int value;

    /** Creates new MyFloat */
    public MyFloat(int val) {
        assign(val);
    }
    
    public MyFloat(MyFloat val) {
//        System.out.println("MyFloat:Free before:"+Runtime.getRuntime().freeMemory());
        assign(val);
//        System.out.println("MyFloat:Free after:"+Runtime.getRuntime().freeMemory());
    }
    
    public MyFloat assign(MyFloat a) {
        this.value = a.value;
        return this;
    }
    
    public MyFloat assign(int a) {
        this.value = a*MULTIPL;
        return this;
    }
    
    public MyFloat add(int a) {
        value += a*MULTIPL;
        return this;
    }
    
    public MyFloat add(MyFloat a) {
        value += a.value;
        return this;
    }
    
    public MyFloat clone() {
        return new MyFloat(this);
    }
    
    public MyFloat sub(int a) {
        value -= a*MULTIPL;
        return this;
    }
    
    public MyFloat sub(MyFloat a) {
        value -= a.value;
        return this;
    }
    
    public MyFloat mult(int a) {
        value *= a;
        return this;
    }
    
    public MyFloat abs() {
        value = Math.abs(value);
        return this;
    }
    
    public MyFloat mult(MyFloat a) {
        mult(a.toInt());
        return this;
    }
    
    public MyFloat div(int a) {
        value /= a;
        return this;
    }
    
    public MyFloat mod(int a) {
        value %= a*MULTIPL;
        return this;
    }
    
    public boolean equals(MyFloat a) {
        return value == a.value;
    }
    
    public boolean equals(int a) {
        return value == a*MULTIPL;
    }
    
    public boolean greaterThan(MyFloat a) {
        return value > a.value;
    }
    
    public boolean greaterThan(int a) {
        return value > a*MULTIPL;
    }
    
    public int toInt() {
        int mult = 1;
        int valabs = value;
        if (value < 0) {
            valabs = -value;
            mult = -1;
        }
        if (valabs % MULTIPL >= MULTIPL/2)
            return mult*(valabs / MULTIPL+1);
        else return mult*(valabs / MULTIPL);
    }
    
    public byte[] toByteArray() {
        int val = value;
        if (val < 0) val = -val;
        byte[] data = new byte[5];
        data[0] = (byte)(val % 128);
        data[1] = (byte)((val >> 7) % 128);
        data[2] = (byte)((val >> 14) % 128);
        data[3] = (byte)((val >> 21) % 128);
        data[4]  = (byte)((val >> 28) % 128);
        if (val != value) data[4] |= 64;
        return data;
    }
    
    public static MyFloat valueOf(byte[] data) {
        int value = ((data[4]&64) << 28)+(data[3] << 21)+(data[2] << 14)+(data[1] << 7)+data[0];
        if (data[4] > 63) value = -value;
        MyFloat ret = new MyFloat(0);
        ret.value = value;
        return ret;
    }
    
    public String toString() {
        return value/MULTIPL+"."+Math.abs(value)%MULTIPL;
    }
}
