/**

    evMiniPool	-	multiplayer pool game engine for mobile phones
    Copyright (C) 2005, Ernestas Vaiciukevicius (ernisv@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 * 
 */
package com.ev.minipool;

import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.List;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

import com.ev.minipool.conn.ConnectionProvider;
import com.ev.minipool.conn.Connectivity;
import com.ev.minipool.multiplayer.GamePacket;
import com.ev.minipool.multiplayer.GamePacketCallback;
import com.ev.minipool.multiplayer.HandshakeCallback;
import com.ev.minipool.multiplayer.HandshakePacket;
import com.ev.minipool.multiplayer.MultiplayerGame;

/**
 * @author Ernestas Vaiciukevicius 
 *
 */
public class PoolMIDlet extends MIDlet implements CommandListener, GamePacketCallback, HandshakeCallback {
    
    // Reference to device display 
    private Display display;
    protected List mainMenu;
    protected BoardCanvas boardCanvas;
    protected ConnectOptionsForm connOptForm;
    protected MultiplayerGame mgame;
    protected MiniPoolPacket gp;
    protected byte currentPlayer;
    protected Command cmdYes, cmdNo, cmdOk;
    protected Form myNumberForm;
    protected String httpRouter;
    protected int connType;
    protected byte[] score;
    
    /**
     *
     */
    public PoolMIDlet() {        
    }
    
    public String getVersion() {
    	return "0.1.0";
    }
    
    /**
     *
     */
    public void startApp() 
            throws MIDletStateChangeException {
        try {
            System.out.println(MyMath.sin(new MyFloat(3)).toString());
        } catch (Exception e) {
            e.printStackTrace();
        }
        display = Display.getDisplay(this);
        mainMenu = new List("Main menu", List.IMPLICIT);
        mainMenu.append("New game", null);
        mainMenu.append("Connection", null);
        mainMenu.append("About", null);
        mainMenu.append("Exit", null);
        cmdYes = new Command("Yes", Command.OK, 2);
        cmdNo = new Command("No", Command.CANCEL, 1);
        cmdOk = new Command("Ok", Command.OK, 10);
        connOptForm = new ConnectOptionsForm(this);
        connOptForm.setExitDisplayable(mainMenu, display);
        boardCanvas = new BoardCanvas(this);
        mainMenu.setCommandListener(this);
        
        Connectivity.setMyDefaultKey("");
        initConnection(0, "");
        
        display.setCurrent(mainMenu);
    }
    
    public String getHTTPRouterAddr() {
        return httpRouter;
    }
    
    public int getConnType() {
        return connType;
    }
    
    public MultiplayerGame getMultiplayerGame() {
        return mgame;
    }
    
    public void initConnection(int type, String httpRouter) {
                this.httpRouter = httpRouter;
                this.connType = type;
                gp = new MiniPoolPacket((byte)0, null, (byte)0, (byte)0);
                if (mgame != null) mgame.cleanup();
                switch (type) {
                    case 0:
                        mgame = new MultiplayerGame("sms://"+Connectivity.getMyDefaultKey()+":1228");
                        break;
                    default:
                        mgame = new MultiplayerGame("httprouter://"+httpRouter+"/["+Connectivity.getMyDefaultKey()+"]");
                        break;                        
                }
                try {
                    ConnectionProvider cp = null;
                    switch (type) {
                        case 0:
                             cp = Connectivity.getConnectionProvider("sms::1228");
                             break;
                        default:
                             cp = Connectivity.getConnectionProvider("httprouter://"+httpRouter+"/[]", Connectivity.getMyDefaultKey());
                             break;
                    }
                    cp.connect();
                    mgame.addListenConn(cp);
                    mgame.setHandshakeCallback(this);
                } catch (Exception e) {
                    e.printStackTrace();
                }
    }
    
    /**
     *
     */
    public void pauseApp() {
    }
    
    /**
     *
     */
    public void destroyApp(boolean unconditional) 
            throws MIDletStateChangeException {
         notifyDestroyed();
    }
    
    public void startGame(boolean first) {
        score = new byte[mgame.getNumPlayers()];
        for (int i = 0; i < mgame.getNumPlayers(); ++i)
            score[i] = (byte)0;
        if (first) {
            mgame.doHandshake();
            if (mgame.getNumPlayers() > 1) {
                Form wait = new Form("Please wait..");
                wait.append("Connecting other players");
                display.setCurrent(wait);
                currentPlayer = mgame.getId();
            } else {
                display.setCurrent(boardCanvas);
                boardCanvas.getUserMove();
            }
        } else {
            boardCanvas.gameState = GameState.WAITING;
            display.setCurrent(boardCanvas);
            mgame.receiveGamePocket(gp, this);
        }
    }
    
    public void userMove(byte aimingForce, MyFloat angle, byte ball) {
        System.out.println("GP send:id:"+mgame.getId());
        System.out.println("angle:"+angle.toString());
        System.out.println("force:"+aimingForce);
        System.out.println("ball:"+ball);
        
        try {
            mgame.broadcast(new MiniPoolPacket(aimingForce, angle, ball, mgame.getId()));
        } catch (Exception e) {
            Error.log(e, this);
        }
    }
    
    public void moveEnded() {
        score[currentPlayer] += boardCanvas.getMovePoints();
        if (mgame.getId() == currentPlayer) {
            boardCanvas.setPoints(boardCanvas.getPoints()+boardCanvas.getMovePoints());
        }
        if (boardCanvas.gameEnded()) {
            Form results = new Form("Game over!");
            results.append("Results\n");
            String[] names = new String[mgame.getNumPlayers()];
            for (int i = 0; i < names.length; ++i) 
                names[i] = mgame.getPlayerName(i);
            for (int i = 0; i < score.length; ++i) 
                for (int j = i+1; j < score.length; ++j) 
                    if (score[i] > score[j]) {
                        byte tmp = score[i];
                        score[i] = score[j];
                        score[j] = tmp;
                        String tmp2 = names[i];
                        names[i] = names[j];
                        names[j] = tmp2;
                    }
            for (int i = 0; i < score.length; ++i)
                results.append(names[i]+" - "+score[i]);
            results.addCommand(cmdOk);
            results.setCommandListener(this);
            display.setCurrent(results);
            initConnection(connType, httpRouter);
        } else if (mgame.getPrevPlayer() != currentPlayer) {
            mgame.receiveGamePocket(gp, this);
        } else {
            currentPlayer = mgame.getId();
            boardCanvas.getUserMove();
        }
    }
    
    public void commandAction(Command c, Displayable d) {
        if (c == List.SELECT_COMMAND) {
            switch (mainMenu.getSelectedIndex()) {
                case 0:
                    display.setCurrent(new StartForm(this, mgame));
                    break;
                case 1:
                    connOptForm.setURL(httpRouter);
                    display.setCurrent(connOptForm);
                    break;
                case 2:
                	Form aboutForm = new Form("About");
                	aboutForm.append("evMiniPool v" + getVersion()+"\n");
                    aboutForm.append("(C) Ernestas Vaiciukevicius, 2003\n");
                    aboutForm.append("ernisv@gmail.com\n");
                    aboutForm.append("License: GPL\n");
                    aboutForm.addCommand(cmdOk);
                    aboutForm.setCommandListener(this);
                    display.setCurrent(aboutForm);
                    break;
                default:
                    try {
                        destroyApp(true);
                    } catch (Exception e) {};
                    break;
            }
        } else if (c == cmdYes) {
            mgame.acceptGame();
            startGame(false);
        } else if (c == cmdNo) {
            mgame.rejectGame();
            display.setCurrent(mainMenu);
        } else if (c == cmdOk) {
            display.setCurrent(mainMenu);
        }
    }
    
    public void gamePacketReceived(GamePacket gpp) {
        MiniPoolPacket gp = (MiniPoolPacket)gpp;
        System.out.println("GamePocketCallback:from:"+gp.getOriginatorId());
//        if (gp.getOriginatorId() != currentPlayer)
//            mgame.receiveGamePocket(gp, this);
//        else {
            currentPlayer = gp.getOriginatorId();
            boardCanvas.showMove(gp.getAimingForce(), gp.getAngle(), gp.getBall());
//        }
    }
    
    public void handshakeReceived(HandshakePacket hp, String refused) {
        if (hp != null) {
            System.out.println("HandshakeCallback...");
            Form join = new Form("Join game");
            join.append("You are invited into a new game.");
            join.append("Do you want to play?");
            join.addCommand(cmdYes);
            join.addCommand(cmdNo);
            join.setCommandListener(this);
            display.setCurrent(join);
        } else if (refused == null) {
            display.setCurrent(boardCanvas);
            boardCanvas.getUserMove();
        } else if (refused != null) {
            initConnection(connType, httpRouter);
            display.setCurrent(new Alert("Refused", "Some player does not want to play", null, AlertType.WARNING), mainMenu);
        }
    }
    
}
