/*
 * TimeThread.java

    evMiniPool	-	multiplayer pool game engine for mobile phones
    Copyright (C) 2005, Ernestas Vaiciukevicius (ernisv@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *
 * Created on Antradienis, 2003, Vasario 25, 15.13
 */

package com.ev.minipool;

import java.util.*;

/**
 * @author Ernestas Vaiciukevicius 
 *
 */
public class TimeThread extends Thread {
    protected Vector objects;
    protected boolean stopped, paused;
    protected int interval;

    /** Creates new TimeThread */
    public TimeThread(int interval) {
        objects = new Vector();
        this.setPriority(Thread.MAX_PRIORITY);
        stopped = false;
        paused = false;
        this.interval = interval;
    }
    
    public void setInterval(int interval) {
        this.interval = interval;
    }
    
    public void addTimeListener(ITimeListener tl) {
        synchronized (objects) {
            objects.addElement(tl);
        }
    }
    
    public void removeTimeListener(ITimeListener tl) {
        synchronized (objects) {
            objects.removeElement(tl);
        }
    }
    
    public void pauseTime() {
        paused = true;
    }
    
    public void resumeTime() {
        synchronized (this) {
            paused = false;
            this.notify();
        }
    }
    
    public void run() {
        long startTime, endTime;
        Enumeration e;
        while (!stopped) {
            if (paused) 
                synchronized (this) {
                    while (paused) 
                        try {
                            this.wait();                        
                        } catch (InterruptedException ex) {
                            if (stopped) return;
                        }
                }
            startTime = System.currentTimeMillis();
            synchronized (objects) {
                for  (e = objects.elements(); e.hasMoreElements(); )
                    ((ITimeListener)(e.nextElement())).timeTick();
            };
            endTime = System.currentTimeMillis();
            if (endTime-startTime < interval)
                try {
                    this.sleep(interval-endTime+startTime);
                } catch (InterruptedException ex) {};
            
        }
    }

}
