/*
 * HttpConnectionProvider.java

    evMiniPool	-	multiplayer pool game engine for mobile phones
    Copyright (C) 2005, Ernestas Vaiciukevicius (ernisv@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *
 * Created on Antradienis, 2003, Kovo 25, 10.41
 */

package com.ev.minipool.conn;

import java.io.DataInputStream;
import java.io.IOException;

import javax.microedition.io.Connector;
import javax.microedition.io.HttpConnection;

/**
 * @author Ernestas Vaiciukevicius 
 *
 */
public class HttpConnectionProvider implements ConnectionProvider {
    protected String connStr;
    protected HttpConnectionListener listenThread;
    protected String myKey, targetKey;
    protected String httpConnStr;
    
    
    public HttpConnectionProvider(String connString, String myKey) throws IOException {
        connStr = connString;
        this.myKey = myKey;
        listenThread = null;
    }
    
    public ReceiveListener getReceiveListener() {
        if (listenThread == null)
            return null;
        else return listenThread.getListener();
    }
    
    public byte[] receive() throws IOException {
        System.out.println("http:receiving:"+myKey);
        HttpConnection conn = (HttpConnection)Connector.open(httpConnStr+"router.receive.php?clientkey="+myKey);
        conn.setRequestMethod(HttpConnection.GET);
        DataInputStream is;
        is = conn.openDataInputStream();
        byte[] buf = null;
        if (conn.getLength() > 0){
            buf = new byte[(int)conn.getLength()];
            is.read(buf);
        };
        is.close();
        conn.close();
        return buf;
    }
    
    protected String encodeData(byte[] data) {
        StringBuffer str = new StringBuffer();
        for (int i = 0; i < data.length; ++i) {
            String hex = Integer.toHexString(data[i]);
            if (hex.length() < 2) hex = "0"+hex;
            str.append("%"+hex);
        }
        return str.toString();
    }
    
    public void send(byte[] data) throws IOException {
        System.out.println("http:sending to "+targetKey+" ["+new String(data)+"]");
        HttpConnection conn = (HttpConnection)Connector.open(httpConnStr+"router.send.php?clientkey="+myKey+"&targetkey="+targetKey+"&data="+encodeData(data));
        conn.setRequestMethod(HttpConnection.GET);
        DataInputStream is;
        is = conn.openDataInputStream();
        is.close();
        conn.close();
    }
    
    public void setReceiveListener(ReceiveListener listener) {
        if (listenThread != null) {
            listenThread.stopListener();
        }
        listenThread = null;
        if (listener != null) {
            listenThread = new HttpConnectionListener(listener, this);
            listenThread.setPriority(Thread.MIN_PRIORITY);
            listenThread.start();
        }
    }
    
    public void disconnect() {
        listenThread.stopListener();
    }
    
    public void connect() throws IOException {
        int beg = connStr.indexOf('[');
        int end = connStr.lastIndexOf(']');
        targetKey = (end-beg>1) ? connStr.substring(beg+1, end) : "";
        httpConnStr = "http:"+connStr.substring(11, beg);
        System.out.println("http:parsed:httpConn:("+httpConnStr+") targetkey:("+targetKey+")");
    }
    
    public String getConnectionString() {
        return connStr;
    }
    
    public void resumeListener() {
        if (listenThread != null) listenThread.resumeListener();
    }
    
    public void suspendListener() {
        if (listenThread != null) listenThread.suspendListener();
    }
    
}
