/*
 * SmsConnectionProvider.java
 
    evMiniPool	-	multiplayer pool game engine for mobile phones
    Copyright (C) 2005, Ernestas Vaiciukevicius (ernisv@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *
 * Created on Antradienis, 2003, Kovo 25, 14.48
 */

package com.ev.minipool.conn;

import java.io.IOException;

import javax.microedition.io.Connector;
import javax.microedition.io.Datagram;
import javax.microedition.io.DatagramConnection;

/**
 * @author Ernestas Vaiciukevicius 
 *
 */
public class SmsConnectionProvider implements ConnectionProvider {
    protected DatagramConnection conn;
    protected String connStr;
    protected SmsConnectionListener listenThread;
    
    
    public SmsConnectionProvider(String connString) throws IOException {
        connStr = connString;
        listenThread = null;
    }
    
    public void connect() throws IOException {
        int ind;
        if ((ind = connStr.indexOf("::")) != -1) {  // if listen to port
            conn = SmsListenConnections.forPort(Integer.parseInt(connStr.substring(ind+2)));
        } else if ((ind = connStr.indexOf("://:")) != -1) {
            conn = SmsListenConnections.forPort(Integer.parseInt(connStr.substring(ind+4)));
        } else conn = (DatagramConnection)Connector.open(connStr, Connector.READ_WRITE, true);
    }
    
    public ReceiveListener getReceiveListener() {
        if (listenThread == null)
            return null;
        else return listenThread.getListener();
    }
    
    public byte[] receive() throws IOException {
        if (getReceiveListener() == null) {
            Datagram dg = conn.newDatagram(0);
            conn.receive(dg);
            return dg.getData();
        } else return null;
    }
    
    public void send(byte[] data) throws IOException {
        Datagram dg = conn.newDatagram(data, data.length);
        conn.send(dg);
    }
    
    public void setReceiveListener(ReceiveListener listener) {
        if (listenThread != null) {
            if (listener == null) 
                listenThread.stopListener();
            else listenThread.changeListener(listener);
        } else if (listener != null) {
            listenThread = new SmsConnectionListener(listener, conn);
            listenThread.setPriority(Thread.MIN_PRIORITY);
            listenThread.start();
        }
    }
    
    
    public void disconnect() {
        listenThread.stopListener();
        try { 
            conn = null;
        } catch (Exception e) {
            System.out.println("Error while closing connection...");
            e.printStackTrace();
        };
        System.out.println("Disconnected...");
    }
    
    public String getConnectionString() {
        return connStr;
    }
    
    public void suspendListener() {};
    
    public void resumeListener() {};
}
