/*
 * HandshakePocket.java

    evMiniPool	-	multiplayer pool game engine for mobile phones
    Copyright (C) 2005, Ernestas Vaiciukevicius (ernisv@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *
 * Created on Ketvirtadienis, 2003, Balandzio 3, 16.20
 */

package com.ev.minipool.multiplayer;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

/**
 * @author Ernestas Vaiciukevicius 
 *
 */
public class HandshakePacket {
    protected String[] connStrs;
    protected byte playAfter;
    protected byte nextPlayer;
    protected byte numPlayers;
    protected byte myId;
    protected byte sign1, sign2;
    
    public HandshakePacket() {
        sign1 = 'h'; sign2 = 'p';
    }
    
    public HandshakePacket(String[] connStrs, byte myId, byte num) {
        this.connStrs = connStrs;
        this.myId = myId;
        this.numPlayers = num;
        sign1 = 'h'; sign2 = 'p';
    }
    
    public String[] getConnStrs() {
        return connStrs;
    }
    
    public int getNumPlayers() {
        return numPlayers;
    }
    
    public void setNextPlayer(byte next) {
        this.nextPlayer = next;
    }

    public void setPrevPlayer(byte prev) {
        this.playAfter = prev;
    }
    
    public void setMyID(byte id) {
        myId = id;
    }
    
    public byte getNextPlayer() {
        return this.nextPlayer;
    }

    public byte getPrevPlayer() {
        return this.playAfter;
    }
    
    public byte getMyID() {
        return myId;
    }
    
    public byte[] parse(byte[] data) throws Exception {
        ByteArrayInputStream is = new ByteArrayInputStream(data);
        if (((byte)is.read()) != sign1 || ((byte)is.read()) != sign2) throw new Exception("Pocket format exception");
        numPlayers = (byte)is.read();
        playAfter = (byte)is.read();
        nextPlayer = (byte)is.read();
        myId = (byte)is.read();
        connStrs = new String[numPlayers];
        for (int i = 0; i < numPlayers; ++i) {
            int len = is.read();
            byte buf[] = new byte[len];
            is.read(buf, 0, len);
            connStrs[i] = new String(buf);
        }
        byte[] buf = new byte[is.available()];
        is.read(buf, 0, is.available());
        try { is.close(); } catch (Exception e) {};
        return buf;
    }
    
    public byte[] toByteArray() {
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        try {
        	os.write(sign1);
        	os.write(sign2);
        	os.write(numPlayers);
        	os.write(playAfter);
        	os.write(nextPlayer);
        	os.write(myId);
        	for (int i = 0; i < numPlayers; ++i) {
        		byte[] buf = connStrs[i].getBytes();
        		os.write(buf.length);
        		os.write(buf, 0, buf.length);
        	}
        } catch (Exception e) {
			// ignore
		}
        byte[] ret = os.toByteArray();
        try { os.close(); } catch (Exception e) {};
        return ret;
    }
}
