/*
 * MultiplayerGame.java

    evMiniPool	-	multiplayer pool game engine for mobile phones
    Copyright (C) 2005, Ernestas Vaiciukevicius (ernisv@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *
 * Created on Ketvirtadienis, 2003, Balandzio 3, 16.19
 */

package com.ev.minipool.multiplayer;

import java.io.IOException;
import java.util.Vector;

import com.ev.minipool.conn.ConnectionProvider;
import com.ev.minipool.conn.Connectivity;
import com.ev.minipool.conn.ReceiveListener;

/**
 * @author Ernestas Vaiciukevicius 
 *
 */
public class MultiplayerGame implements ReceiveListener {
    protected Vector conns;
    protected byte myId, agreeLeft;
    protected String myConnStr;
    protected boolean handshakeDone, handshakeWait, blocked;
    protected Vector cache;
    protected byte playAfter, nextPlayer;
    protected GamePacketCallback callback;
    protected GamePacket callback_arg;
    protected HandshakeCallback handshake_callback;
    protected Vector listenConns;
    protected HandshakePacket receivedHandshake;

    public MultiplayerGame(String myConnStr) {
        myId = 0;
        this.myConnStr = myConnStr;
        conns = new Vector();
        conns.addElement(new Object());
        cache = new Vector();
        nextPlayer = 0;
        playAfter = 0;
        handshakeDone = false;
        handshakeWait = false;
        blocked = false;
        callback = null;
        handshake_callback = null;
        listenConns = new Vector();
        receivedHandshake = null;
    }
    
    public void cleanup() {
        conns.removeAllElements();
        conns.addElement(new Object());
        cache.removeAllElements();
        nextPlayer = 0;
        playAfter = 0;
        handshakeDone = false;
        handshakeWait = false;
        blocked = false;
        callback = null;
        handshake_callback = null;
        for (int i = 0; i < listenConns.size(); ++i) 
            ((ConnectionProvider)listenConns.elementAt(i)).disconnect();
        listenConns.removeAllElements();
        receivedHandshake = null;
        System.gc();
    }
    
    public final String getPlayerName(int id) {
        String connStr = (id == myId) ? myConnStr : ((ConnectionProvider)(conns.elementAt(id))).getConnectionString();
        int begIndex;
        int endIndex;
        if (connStr.startsWith("sms")) {
            begIndex = connStr.indexOf("//")+2;
            endIndex = (connStr.indexOf("/", begIndex) == -1) ? ((connStr.indexOf(":", begIndex) == -1) ? connStr.length() : connStr.indexOf(":", begIndex)) : connStr.indexOf("/", begIndex);
        } else {
            begIndex = connStr.indexOf("[")+1;
            endIndex = connStr.indexOf("]");
        }
        return connStr.substring(begIndex, endIndex);
    }
    
    public byte getPrevPlayer() {
        return playAfter;
    }
    
    public byte getNextPlayer() {
        return nextPlayer;
    }
    
    public int getNumPlayers() {
        return conns.size();
    }
    
    public void addPlayerConn(ConnectionProvider conn) {
        conns.addElement(conn);
        playAfter = (byte)(conns.size()-1);
    }
    
    public void broadcast(GamePacket gp) throws IOException {
        broadcast(gp.toByteArray());
    }
    
    public void broadcast(byte[] data) throws IOException {
        for (int i = 0; i < conns.size(); ++i) 
            if (i != myId)
            {
//                System.out.println("broadcast:to:"+i+"el:"+(conns.elementAt(i)).toString());
                ((ConnectionProvider)(conns.elementAt(i))).send(data);
            }
    }
    
    public byte getId() {
        return myId;
    }
    
    public void setHandshakeCallback(HandshakeCallback callback) {
        handshake_callback = callback;
    }
    
    public void doHandshake() {
        System.out.println("Beginning handshake...");
        String[] connStrs = new String[conns.size()];
        connStrs[0] = myConnStr;
        for (int i = 1; i < conns.size(); ++i) {
            connStrs[i] = ((ConnectionProvider)(conns.elementAt(i))).getConnectionString();
        }
        
        HandshakePacket hp = new HandshakePacket(connStrs, (byte)0, (byte)(conns.size()));
        for (int i = 1; i < conns.size(); ++i) {
            hp.setMyID((byte)i);
            hp.setPrevPlayer((byte)(i-1));
            hp.setNextPlayer((byte)((i+1) % conns.size()));
            try {
                send(hp.toByteArray(), (byte)i);
            } catch (IOException e) {};
        }
        agreeLeft = (byte)(conns.size()-1);
        handshakeWait = true;
    }
    
    public void send(GamePacket pocket, byte destId) throws IOException {
        send(pocket.toByteArray(), destId);
    }
    
    public void send(byte[] data, byte destId) throws IOException {
        ((ConnectionProvider)conns.elementAt(destId)).send(data);
        System.gc();
    }
    
    public boolean dataAvailable() {
//        System.out.println("available:"+cache.size());
        return cache.size() > 0;
    }
    
    public synchronized void receiveGamePocket(GamePacket gp, GamePacketCallback callback) {
        if (cache.size() > 0) {
            boolean badPocket = true;
            while (badPocket && cache.size() > 0) {
                badPocket = false;
                try { 
                    gp.parse((byte[])(cache.firstElement()));
                } catch (Exception e) {
                    badPocket = true;
                };
                cache.removeElementAt(0);
            }
            if (!badPocket) callback.gamePacketReceived(gp);
        } else {
            callback_arg = gp;
            this.callback = callback;
        }
    }

    
    public void addListenConn(ConnectionProvider conn) {
        listenConns.addElement(conn);
        conn.setReceiveListener(this);
    }
    
    public void suspendListeners() {
        for (int i = 0; i <  listenConns.size(); ++i)
            ((ConnectionProvider)listenConns.elementAt(i)).suspendListener();
    }
    
    public void resumeListeners() {
        for (int i = 0; i <  listenConns.size(); ++i)
            ((ConnectionProvider)listenConns.elementAt(i)).resumeListener();
    }
    
    public void rejectGame() {
        if (receivedHandshake != null) {
            byte[] reply = new byte[4];
            reply[0] = 'h';
            reply[1] = 'r';
            reply[2] = receivedHandshake.getMyID();
            reply[3] = 0;
            try {
                ConnectionProvider c = Connectivity.getConnectionProvider((receivedHandshake.getConnStrs())[0]);
                c.connect();
                c.send(reply);
            } catch (IOException e) {
                e.printStackTrace();
            }
            receivedHandshake = null;
            handshakeDone = false;
            blocked = false;
        }
    }

    public void acceptGame() {
        if (receivedHandshake != null) {
            conns.removeAllElements();
            myId = receivedHandshake.getMyID();
            playAfter = receivedHandshake.getPrevPlayer();
            nextPlayer = receivedHandshake.getNextPlayer();
            String[] connStrs = receivedHandshake.getConnStrs();
            for (int i = 0; i < receivedHandshake.getNumPlayers(); ++i) 
                  if (i != myId) {
                      try {
                           conns.addElement(Connectivity.getConnectionProvider(connStrs[i]));
                           ((ConnectionProvider)(conns.lastElement())).connect();
                            } catch (IOException e) {
                                System.out.println(e.toString());
                            };
                  } else conns.addElement(new Object());
            byte reply[] = new byte[4];
            reply[0] = 'h';
            reply[1] = 'r';
            reply[2] = myId;
            reply[3] = 1;
            try {
                send(reply, (byte)0);
            } catch (IOException e) {
                e.printStackTrace();
            }
            receivedHandshake = null;
            handshakeDone = true;
            blocked = false;
            System.out.println("Handshake done");
        }
    }
    
    public void dataReceived(byte[] data) {
        System.gc();
        if (handshakeDone) {
            synchronized (this) {
                boolean isGP = true;
                try {
                    callback_arg.parse(data);
                } catch (Exception e) {
                    isGP = false;
                }
                if (callback != null && isGP) {
                    GamePacketCallback tmp = callback;
                    callback = null;
//        System.out.println("GP received:id:"+callback_arg.getOriginatorId());
//        System.out.println("angle:"+((MiniPool.MiniPoolPocket)callback_arg).getAngle().toString());
//        System.out.println("force:"+((MiniPool.MiniPoolPocket)callback_arg).getAimingForce());
//        System.out.println("ball:"+((MiniPool.MiniPoolPocket)callback_arg).getBall());
                    tmp.gamePacketReceived(callback_arg);
                } else cache.addElement(data);
            };
            System.out.println("Received data:"+new String(data));
        } else if (handshakeWait) {
            if (data[0] == 'h' && data[1] == 'r') {
                if (data[3] == 1) {
                    --agreeLeft;
                    if (agreeLeft == 0 && handshake_callback != null) { 
                         handshakeDone = true;
                         handshakeWait = false;
                         handshake_callback.handshakeReceived(null, null);
                    }
                } else handshake_callback.handshakeReceived(null, ((ConnectionProvider)(conns.elementAt(data[2]))).getConnectionString());
            }
        } else if (!blocked) {
            HandshakePacket hp = new HandshakePacket();
            boolean isHP = true;
            try {
                hp.parse(data);
            } catch (Exception e) {
                isHP = false;
            }
            if (handshake_callback != null && isHP) {
                blocked = true;
                receivedHandshake = hp;
                handshake_callback.handshakeReceived(hp, null);
            }
        }
    }
}
